﻿//---------------------------------------------
// Initialization
//#region Initialization

var fs = require('fs');
var Logger;// We'll inherit this during initialization

var SmtpDefault = {
    "Default": true,
    "HostName": "Undefined",
    "Enabled" : false,
    
    "Transport": {
        "Port": 465,
        "Host": "127.0.0.1",
        "Secure": true,
        "Auth": {
            "User": "Undefined",
            "Pass": "Undefined"
        }
    },
    
    "Options": {
        "From": "Undefined@Undefined.com",
        "To": [{}],
        "Subject": "Event from Summit(Crest)",
        "Text": "Undefined"
    }
}

var SmtpSettings = [];
exports.SmtpSettings = SmtpSettings;// Make settings visible to external modules
var Mailer = [];

var _this = this;// To allow access to exported functions

exports.InitializeSmtp = function (nodemailer, logger, callback) {
    Logger = logger;// Use this as the logging engine
    Mailer = nodemailer;// Use this as a connection to the NodeMailer library
    
    // Read the smtp settings
    try {
        SmtpSettings = require("./smtp.json");
    }
    catch (error) {
        Logger.error("SMTP.InitializeSmtp: Failed to read SMTP settings, initializing to default. Error=" + error);
        SmtpSettings = SmtpDefault;
    }
    
    // This is so we can print logger information, but without password as per Mantis 874
    var SmtpSettingsNoPass = JSON.parse(JSON.stringify(SmtpSettings));
    SmtpSettingsNoPass.Transport.Auth.Pass = "**********";
    Logger.info("SMTP.InitializeSmtp: SMTP settings initialized - " + JSON.stringify(SmtpSettingsNoPass));
    
    // Return SmtpSettings
    callback(SmtpSettings);
}
//#endregion Initialization

//---------------------------------------------
// SMTP Operations
//#region SMTP Operations
exports.GetSmtpSettings = function (callback) {
    Logger.info("SMTP.SetSmtpSettings: Retreiving SMTP settings - " + SmtpSettings);
    callback(SmtpSettings);// Send back the last known good value
}

// Write the SMTP settings to our SMTP settings file
exports.SetSmtpSettings = function (smtp, callback) {
    SmtpSettings = smtp;// We may have new data, so use it
    try {
        fs.truncate("./smtp.json", 0, function () {
            fs.writeFile("./smtp.json", JSON.stringify(SmtpSettings, null, 2), function (err) {
                if (err) {
                    Logger.error("SMTP.SetSmtpSettings: Failed to write SMTP settings. Error=" + err);
                }
                var password = SmtpSettings.Transport.Auth.Pass;
                SmtpSettings.Transport.Auth.Pass = "HIDDEN";
                Logger.info("SMTP.SetSmtpSettings: Successfully wrote SMTP settings - " + JSON.stringify(SmtpSettings));
                SmtpSettings.Transport.Auth.Pass = password;
            });
        });
    } 
    catch (error) {
        Logger.error("SMTP.SetSmtpSettings: Exception in wriring SMTP settings. Error=" + error);
    }
    finally {
        callback(SmtpSettings);// Return the current state of affairs
    }
}

exports.SendSmtp = function (message, ioInfo, event, callback) {
    Logger.debug("SMTP.SendSmtp: Sending SMTP Event for " + ioInfo + " - message=" + message);

    // Don't send emails if we haven't set our SMTP properly, any Put of SMTP info will clear this or
    // Don't send emails if disabled, user must explicitly enable SMTP
    if ((SmtpSettings.Default == true) || (SmtpSettings.Enabled == false)){
        callback({ "result": false });
        return
    }
    
    // Create reusable transporter object using the default SMTP transport
    var transporter = Mailer.createTransport({
        port: SmtpSettings.Transport.Port,
        host: SmtpSettings.Transport.Host,
        secure: SmtpSettings.Transport.Secure,
        auth: {
            user: SmtpSettings.Transport.Auth.User,
            pass: SmtpSettings.Transport.Auth.Pass
        }
    });
    Logger.trace("SMTP.SendSmtp: Sending SMTP Event for " + ioInfo + " - transporter=" + JSON.stringify(transporter));
    
    // Setup the To list
    var ToList = "";
    var AddedEmail = false;
    for (var t = 0; t < SmtpSettings.Options.To.length; t++) {
        if (SmtpSettings.Options.To[t].Enabled) {
            Logger.trace("SMTP.SendSmtp: Adding email=" + SmtpSettings.Options.To[t].Email);
            if (AddedEmail) ToList += ",";// Add separator if we previously had an email
            ToList += SmtpSettings.Options.To[t].Email;
            AddedEmail = true;
        }
    }
    Logger.info("SMTP.SendSmtp: Sending SMTP Event for " + ioInfo + " - To: " + ToList);

    // Setup e-mail data with unicode symbols
    var mailOptions = {
        from: SmtpSettings.Options.From, // sender address
        to: ToList, // list of receipients
        subject: "Summit (" + SmtpSettings.HostName + ") Event: " + event, // Subject line
        text: message, // plaintext body
    };
    Logger.trace("SMTP.SendSmtp: Sending SMTP Event for " + ioInfo + " - mailOptions=" + JSON.stringify(mailOptions));
    
    // send mail with defined transport object
    var result = false;
    transporter.sendMail(mailOptions, function (error, info) {
        if (error) {
            Logger.error("SMTP.SendSmtp: Send email for " + ioInfo + " - error=" + error);
            result = false;
            callback({ "result": result });
        } else {
            Logger.info("SMTP.SendSmtp: Send email for " + ioInfo + " - result=" + info.response);
            result = true;
            callback({ "result": result });
        }
    });    
}
//#endregion SMTP Operations